/*
 * Created on Oct 6, 2004
 * 
 * Copyright 2004 Harshy Wanigasekara
 * This software distributed under the terms of the GNU Public license.
 * See the accompanying file COPYING.txt for complete information.
 *
 * This class contains the values and algorithms to perform
 * the calculations.
 * 
 */

import java.lang.String;
import net.jscience.math.kvm.MathFP;

/**
 * @author harshy
 * @version 1.0
 *
 * Calculation class.
 * Takes inputs and produces result.  Both inputs and
 * result are strings.
 * 
 * This class utilizes the net.jscience.math.kvm.MathFP
 * floating point library from Onno Hommes at
 * Science Technologies:
 * http://home.rochester.rr.com/ohommes/MathFP/
 * Currently uses the default precision setting for the
 * fixed point values.
 * 
 */
public class Calculation
{
	/**
	 * Input value: price of the meal.
	 * This is a fixed point value.
	 */
	int mealPrice = 0;
	
	/**
	 * Input value: Number of people amount whom to
	 *     divide the total amount.
	 * This is a fixed point value.
	 */
	int numPeople = MathFP.toFP(1);
	
	/**
	 * Input value: Tip percentage.
	 * This is a fixed point value.
	 */
	int tipPercent = MathFP.toFP("0.15");
	
	/**
	 * Output value: Dollar amount of tip.
	 * this is a fixed point value.
	 */
	int tipAmount = 0;
	
	/**
	 * Output value: Total of tip amount and meal amount.
	 * This is a fixed point value.
	 */
	int totalAmount = 0;
	
	/**
	 * Output value: Price per person.
	 * This is a fixed point value.
	 */
	int perPerson = 0;
	
	/**
	 * Constant, used for rounding to nearest cent.
	 */
	static final int roundingConst = MathFP.toFP("0.005");
	
	public Calculation()
	{
	}
	
	/**
	 * This method recalculates the output values.
	 * It is called whenever an input changes.
	 */
	private void reCalculate()
	{
		if (numPeople != 0)
		{
			tipAmount = MathFP.mul(mealPrice, tipPercent);
			totalAmount = tipAmount + mealPrice;
			perPerson = MathFP.div(totalAmount, numPeople);
		}
	}
	
	/**
	 * Set the price of the meal.
	 * @param newPrice String: The price of the meail as entered
	 *        by the user.
	 */
	public void setMealPrice(String newPrice)
	{
		try
		{
			mealPrice = MathFP.toFP(newPrice);
		}
		catch (Exception e)
		{
			mealPrice = 0;
		}
		reCalculate();
	}
	
	/**
	 * Set the number of people to divide the total amount by.
	 * @param newPeople String: The number of people as
	 *        entered by the user.
	 */
	public void setNumPeople(String newPeople)
	{
		try
		{
			numPeople = MathFP.toFP(newPeople);
		}
		catch (Exception e)
		{
			numPeople = MathFP.toFP("1");
		}
		reCalculate();
	}
	
	/**
	 * Set the tip percent value.  Takes a whole number
	 * between 0 and 100.
	 * @param newPercent String: The tip percentage entered
	 *        as a whole number by the user.
	 */
	public void setTipPercent(String newPercent)
	{
		try
		{
			tipPercent = MathFP.div(MathFP.toFP(newPercent),
					MathFP.toFP("100"));
		}
		catch (Exception e)
		{
			tipPercent = 0;
		}
		reCalculate();
	}
	
	/**
	 * Return the tip amount as a displayable value.
	 * @return String: The tip amount.
	 */
	public String getTipAmount()
	{
		return MathFP.toString(tipAmount + roundingConst, 2);
	}
	
	/**
	 * Return the total amount as a displayable value.
	 * @return String: The total amount.
	 */
	public String getTotalAmount()
	{
		return MathFP.toString(totalAmount + roundingConst, 2);
	}
	
	/**
	 * Return the per person contribution to the meal.
	 * @return String: The per person value.
	 */
	public String getPerPerson()
	{
		return MathFP.toString(perPerson + roundingConst, 2);
	}
}
